"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObservableQueryConnection = void 0;
class ObservableQueryConnection {
    constructor(_route) {
        this._route = _route;
        this._disconnected = false;
    }
    connect(dataReceived) {
        const secure = document.location.protocol.indexOf('https') === 0;
        const url = `${secure ? 'wss' : 'ws'}://${document.location.host}${this._route}`;
        let timeToWait = 500;
        const timeExponent = 500;
        const retries = 100;
        let currentAttempt = 0;
        const connectSocket = () => {
            const retry = () => {
                currentAttempt++;
                if (currentAttempt > retries) {
                    console.log(`Attempted ${retries} retries for route '${this._route}'. Abandoning.`);
                    return;
                }
                console.log(`Attempting to reconnect for '${this._route}' (#${currentAttempt})`);
                setTimeout(connectSocket, timeToWait);
                timeToWait += (timeExponent * currentAttempt);
            };
            this._socket = new WebSocket(url);
            this._socket.onopen = (ev) => {
                console.log(`Connection for '${this._route}' established`);
                timeToWait = 500;
                currentAttempt = 0;
            };
            this._socket.onclose = (ev) => {
                if (this._disconnected)
                    return;
                console.log(`Unexpected connection closed for route '${this._route}`);
                retry();
            };
            this._socket.onerror = (error) => {
                console.log(`Error with connection for '${this._route} - ${error}`);
                retry();
            };
            this._socket.onmessage = (ev) => {
                dataReceived(JSON.parse(ev.data));
            };
        };
        connectSocket();
    }
    disconnect() {
        var _a;
        console.log(`Disconnecting '${this._route}'`);
        this._disconnected = true;
        (_a = this._socket) === null || _a === void 0 ? void 0 : _a.close();
    }
}
exports.ObservableQueryConnection = ObservableQueryConnection;
//# sourceMappingURL=ObservableQueryConnection.js.map